@@ -1,5 +1,5 @@
 /* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2006 Ciaran Anscomb <evilwm@6809.org.uk>
+ * Copyright (C) 1999-2007 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
 #include <stdlib.h>
@@ -333,12 +333,18 @@ void event_main_loop(void) {
 				handle_unmap_event(&ev.xunmap); break;
 			case MappingNotify:
 				handle_mappingnotify_event(&ev.xmapping); break;
-#ifdef SHAPE
 			default:
+#ifdef SHAPE
 				if (have_shape && ev.type == shape_event) {
 					handle_shape_event((XShapeEvent *)&ev);
 				}
 #endif
+#ifdef RANDR
+				if (have_randr && ev.type == randr_event_base + RRScreenChangeNotify) {
+					XRRUpdateConfiguration(&ev);
+				}
+#endif
+				break;
 		}
 		if (need_client_tidy) {
 			Client *c, *nc;
