@@ -1,3 +1,7 @@
+/* evilwm - Minimalist Window Manager for X
+ * Copyright (C) 1999-2007 Ciaran Anscomb <evilwm@6809.org.uk>
+ * see README for license and other details. */
+
 #include <X11/X.h>
 #include <X11/Xatom.h>
 #include <X11/Xlib.h>
@@ -7,6 +11,13 @@
 #ifdef SHAPE
 #include <X11/extensions/shape.h>
 #endif
+#ifdef RANDR
+#include <X11/extensions/Xrandr.h>
+#endif
+
+#ifndef __GNUC__
+# define  __attribute__(x)
+#endif
 
 #include "keymap.h"
 
@@ -165,6 +176,9 @@ extern ScreenInfo   *screens;
 #ifdef SHAPE
 extern int          have_shape, shape_event;
 #endif
+#ifdef RANDR
+extern int          have_randr, randr_event_base;
+#endif
 
 /* Standard X protocol atoms */
 extern Atom xa_wm_state;
@@ -218,7 +232,7 @@ void set_shape(Client *c);
 
 /* events.c */
 
-void event_main_loop(void);
+void event_main_loop(void) __attribute__((noreturn));
 
 /* misc.c */
 
