@@ -1,5 +1,5 @@
 /* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2006 Ciaran Anscomb <evilwm@6809.org.uk>
+ * Copyright (C) 1999-2007 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
 #include <stdlib.h>
@@ -20,6 +20,9 @@ ScreenInfo  *screens;
 #ifdef SHAPE
 int         have_shape, shape_event;
 #endif
+#ifdef RANDR
+int         have_randr, randr_event_base;
+#endif
 
 /* Standard X protocol atoms */
 Atom xa_wm_state;
@@ -62,6 +65,31 @@ Client          *head_client = NULL;
 Client          *current = NULL;
 volatile Window initialising = None;
 
+/* Simple command-line options */
+static struct simple_option {
+	const char *opt;
+	enum { OPT_INT, OPT_STRING, OPT_MODIFIERS } type;
+	void *dest;
+} option_list[] = {
+	{ "-fn", OPT_STRING, &opt_font },
+	{ "-display", OPT_STRING, &opt_display },
+	{ "-fg", OPT_STRING, &opt_fg },
+	{ "-bg", OPT_STRING, &opt_bg },
+#ifdef VWM
+	{ "-fc", OPT_STRING, &opt_fc },
+#endif
+	{ "-bw", OPT_INT, &opt_bw },
+	{ "-term", OPT_STRING, &opt_term[0] },
+#ifdef SNAP
+	{ "-snap", OPT_INT, &opt_snap },
+#endif
+	{ "-mask1", OPT_MODIFIERS, &grabmask1 },
+	{ "-mask2", OPT_MODIFIERS, &grabmask2 },
+	{ "-altmask", OPT_MODIFIERS, &altmask },
+};
+
+#define NUM_SIMPLE_OPTS (int)(sizeof(option_list) / sizeof(struct simple_option))
+
 static void setup_display(void);
 static void *xmalloc(size_t size);
 static unsigned int parse_modifiers(char *s);
@@ -71,29 +99,33 @@ int main(int argc, char *argv[]) {
 	int i;
 
 	for (i = 1; i < argc; i++) {
-		if (!strcmp(argv[i], "-fn") && i+1<argc)
-			opt_font = argv[++i];
-		else if (!strcmp(argv[i], "-display") && i+1<argc) {
-			opt_display = argv[++i];
+		if (i+1 < argc) {
+			int done = 0;
+			int j;
+			for (j = 0; j < NUM_SIMPLE_OPTS; j++) {
+				if (0 == strcmp(argv[i], option_list[j].opt)) {
+					i++;
+					switch (option_list[j].type) {
+					case OPT_STRING:
+						*((char **)option_list[j].dest) = argv[i];
+						break;
+					default:
+					case OPT_INT:
+						*((int *)option_list[j].dest) = atoi(argv[i]);
+						break;
+					case OPT_MODIFIERS:
+						*((unsigned int *)option_list[j].dest) = parse_modifiers(argv[i]);
+						break;
+					}
+					done = 1;
+					break;
+				}
+			}
+			if (done)
+				continue;
 		}
-		else if (!strcmp(argv[i], "-fg") && i+1<argc)
-			opt_fg = argv[++i];
-		else if (!strcmp(argv[i], "-bg") && i+1<argc)
-			opt_bg = argv[++i];
-#ifdef VWM
-		else if (!strcmp(argv[i], "-fc") && i+1<argc)
-			opt_fc = argv[++i];
-#endif
-		else if (!strcmp(argv[i], "-bw") && i+1<argc)
-			opt_bw = atoi(argv[++i]);
-		else if (!strcmp(argv[i], "-term") && i+1<argc) {
-			opt_term[0] = argv[++i];
-			opt_term[1] = opt_term[0];
-#ifdef SNAP
-		} else if (!strcmp(argv[i], "-snap") && i+1<argc) {
-			opt_snap = atoi(argv[++i]);
-#endif
-		} else if (!strcmp(argv[i], "-app") && i+1<argc) {
+
+		if (!strcmp(argv[i], "-app") && i+1<argc) {
 			Application *new = xmalloc(sizeof(Application));
 			char *tmp;
 			i++;
@@ -132,15 +164,6 @@ int main(int argc, char *argv[]) {
 			if (head_app)
 				head_app->sticky = 1;
 #endif
-		} else if (!strcmp(argv[i], "-mask1") && i+1<argc) {
-			i++;
-			grabmask1 = parse_modifiers(argv[i]);
-		} else if (!strcmp(argv[i], "-mask2") && i+1<argc) {
-			i++;
-			grabmask2 = parse_modifiers(argv[i]);
-		} else if (!strcmp(argv[i], "-altmask") && i+1<argc) {
-			i++;
-			altmask = parse_modifiers(argv[i]);
 #ifdef SOLIDDRAG
 		} else if (!strcmp(argv[i], "-nosoliddrag")) {
 			solid_drag = 0;
@@ -166,10 +189,14 @@ int main(int argc, char *argv[]) {
 			LOG_INFO("\n              [-nosoliddrag]");
 #endif
 			LOG_INFO(" [-V]\n");
-			exit((!strcmp(argv[i], "-h") || !strcmp(argv[i], "--help"))?0:1);
+			if (0 == strcmp(argv[i], "-h") || 0 == strcmp(argv[i], "--help"))
+				exit(0);
+			exit(1);
 		}
 	}
 
+	opt_term[1] = opt_term[0];
+
 	act.sa_handler = handle_signal;
 	sigemptyset(&act.sa_mask);
 	act.sa_flags = 0;
@@ -277,6 +304,17 @@ static void setup_display(void) {
 		have_shape = XShapeQueryExtension(dpy, &shape_event, &e_dummy);
 	}
 #endif
+	/* Xrandr extension? */
+#ifdef RANDR
+	{
+		int e_dummy;
+		have_randr = XRRQueryExtension(dpy, &randr_event_base, &e_dummy);
+#ifdef DEBUG
+		if (!have_randr)
+			LOG_DEBUG("Xrandr is not supported on this display.\n");
+#endif
+	}
+#endif
 
 	/* now set up each screen in turn */
 	num_screens = ScreenCount(dpy);
@@ -304,6 +342,11 @@ static void setup_display(void) {
 
 		screens[i].screen = i;
 		screens[i].root = RootWindow(dpy, i);
+#ifdef RANDR
+		if (have_randr) {
+			XRRSelectInput(dpy, screens[i].root, RRScreenChangeNotifyMask);
+		}
+#endif
 #ifdef VWM
 		screens[i].vdesk = KEY_TO_VDESK(XK_1);
 #endif
