@@ -1,5 +1,5 @@
 /* evilwm - Minimalist Window Manager for X
- * Copyright (C) 1999-2006 Ciaran Anscomb <evilwm@6809.org.uk>
+ * Copyright (C) 1999-2007 Ciaran Anscomb <evilwm@6809.org.uk>
  * see README for license and other details. */
 
 #include <stdio.h>
@@ -92,6 +92,7 @@ static void draw_outline(Client *c) {
 }
 #endif
 
+#ifdef MOUSE
 static void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2) {
 	c->width = abs(x1 - x2);
 	c->height = abs(y1 - y2);
@@ -105,7 +106,6 @@ static void recalculate_sweep(Client *c, int x1, int y1, int x2, int y2) {
 	c->y = (y1 <= y2) ? y1 : y1 - c->height;
 }
 
-#ifdef MOUSE
 void sweep(Client *c) {
 	XEvent ev;
 	int old_cx = c->x;
@@ -185,16 +185,10 @@ static int absmin(int a, int b) {
 
 static void snap_client(Client *c) {
 	int dx, dy;
+	int dpy_width = DisplayWidth(dpy, c->screen->screen);
+	int dpy_height = DisplayHeight(dpy, c->screen->screen);
 	Client *ci;
 
-	/* snap to screen border */
-	if (abs(c->x - c->border) < opt_snap) c->x = c->border;
-	if (abs(c->y - c->border) < opt_snap) c->y = c->border;
-	if (abs(c->x + c->width + c->border - DisplayWidth(dpy, c->screen->screen)) < opt_snap)
-		c->x = DisplayWidth(dpy, c->screen->screen) - c->width - c->border;
-	if (abs(c->y + c->height + c->border - DisplayHeight(dpy, c->screen->screen)) < opt_snap)
-		c->y = DisplayHeight(dpy, c->screen->screen) - c->height - c->border;
-
 	/* snap to other windows */
 	dx = dy = opt_snap;
 	for (ci = head_client; ci; ci = ci->next) {
@@ -222,6 +216,19 @@ static void snap_client(Client *c) {
 		c->x += dx;
 	if (abs(dy) < opt_snap)
 		c->y += dy;
+
+	/* snap to screen border */
+	if (abs(c->x - c->border) < opt_snap) c->x = c->border;
+	if (abs(c->y - c->border) < opt_snap) c->y = c->border;
+	if (abs(c->x + c->width + c->border - dpy_width) < opt_snap)
+		c->x = dpy_width - c->width - c->border;
+	if (abs(c->y + c->height + c->border - dpy_height) < opt_snap)
+		c->y = dpy_height - c->height - c->border;
+
+	if (abs(c->x) == c->border && c->width == dpy_width)
+		c->x = 0;
+	if (abs(c->y) == c->border && c->height == dpy_height)
+		c->y = 0;
 }
 #endif /* def SNAP */
 
@@ -322,7 +329,6 @@ void maximise_client(Client *c, int hv) {
 			c->height = DisplayHeight(dpy, c->screen->screen);
 		}
 	}
-	recalculate_sweep(c, c->x, c->y, c->x + c->width, c->y + c->height);
 	moveresize(c);
 	discard_enter_events();
 }
@@ -445,31 +451,33 @@ static void grab_keysym(Window w, unsigned int mask, KeySym keysym) {
 	}
 }
 
-void grab_keys_for_screen(ScreenInfo *s) {
-	KeySym *keysym;
-	KeySym keys_to_grab[] = {
-		KEY_NEW, KEY_KILL,
-		KEY_TOPLEFT, KEY_TOPRIGHT, KEY_BOTTOMLEFT, KEY_BOTTOMRIGHT,
-		KEY_LEFT, KEY_RIGHT, KEY_DOWN, KEY_UP,
-		KEY_LOWER, KEY_ALTLOWER, KEY_INFO, KEY_MAXVERT, KEY_MAX,
+static KeySym keys_to_grab[] = {
 #ifdef VWM
-		KEY_FIX, KEY_PREVDESK, KEY_NEXTDESK,
-		XK_1, XK_2, XK_3, XK_4, XK_5, XK_6, XK_7, XK_8,
+	KEY_FIX, KEY_PREVDESK, KEY_NEXTDESK,
+	XK_1, XK_2, XK_3, XK_4, XK_5, XK_6, XK_7, XK_8,
 #endif
-		0
-	};
-	KeySym alt_keys_to_grab[] = {
-		KEY_KILL, KEY_LEFT, KEY_RIGHT, KEY_DOWN, KEY_UP, 0
-	};
+	KEY_NEW, KEY_KILL,
+	KEY_TOPLEFT, KEY_TOPRIGHT, KEY_BOTTOMLEFT, KEY_BOTTOMRIGHT,
+	KEY_LEFT, KEY_RIGHT, KEY_DOWN, KEY_UP,
+	KEY_LOWER, KEY_ALTLOWER, KEY_INFO, KEY_MAXVERT, KEY_MAX
+};
+#define NUM_GRABS (int)(sizeof(keys_to_grab) / sizeof(KeySym))
 
+static KeySym alt_keys_to_grab[] = {
+	KEY_KILL, KEY_LEFT, KEY_RIGHT, KEY_DOWN, KEY_UP
+};
+#define NUM_ALT_GRABS (int)(sizeof(alt_keys_to_grab) / sizeof(KeySym))
+
+void grab_keys_for_screen(ScreenInfo *s) {
+	int i;
 	/* Release any previous grabs */
 	XUngrabKey(dpy, AnyKey, AnyModifier, s->root);
 	/* Grab key combinations we're interested in */
-	for (keysym = keys_to_grab; *keysym; keysym++) {
-		grab_keysym(s->root, grabmask1, *keysym);
+	for (i = 0; i < NUM_GRABS; i++) {
+		grab_keysym(s->root, grabmask1, keys_to_grab[i]);
 	}
-	for (keysym = alt_keys_to_grab; *keysym; keysym++) {
-		grab_keysym(s->root, grabmask1 | altmask, *keysym);
+	for (i = 0; i < NUM_ALT_GRABS; i++) {
+		grab_keysym(s->root, grabmask1 | altmask, alt_keys_to_grab[i]);
 	}
 	grab_keysym(s->root, grabmask2, KEY_NEXT);
 }
